@@ -13,6 +13,7 @@
 # limitations under the License.
 
 import hashlib
+import json
 import shutil
 import os
 import sys
@@ -244,14 +245,33 @@ def browser_type(playwright: Playwright, browser_name: str) -> BrowserType:
     return getattr(playwright, browser_name)
 
 
+@pytest.fixture(scope="session")
+def connect_options() -> Optional[Dict]:
+    return None
+
+
 @pytest.fixture(scope="session")
 def launch_browser(
     browser_type_launch_args: Dict,
     browser_type: BrowserType,
+    connect_options: Optional[Dict],
 ) -> Callable[..., Browser]:
     def launch(**kwargs: Dict) -> Browser:
         launch_options = {**browser_type_launch_args, **kwargs}
-        browser = browser_type.launch(**launch_options)
+        if connect_options:
+            browser = browser_type.connect(
+                **(
+                    {
+                        **connect_options,
+                        "headers": {
+                            "x-playwright-launch-options": json.dumps(launch_options),
+                            **(connect_options.get("headers") or {}),
+                        },
+                    }
+                )
+            )
+        else:
+            browser = browser_type.launch(**launch_options)
         return browser
 
     return launch
@@ -388,7 +408,14 @@ def device(pytestconfig: Any) -> Optional[str]:
     return pytestconfig.getoption("--device")
 
 
-def pytest_addoption(parser: Any) -> None:
+def pytest_addoption(
+    parser: pytest.Parser, pluginmanager: pytest.PytestPluginManager
+) -> None:
+    # Check for incompatible async plugin early
+    if pluginmanager.has_plugin("pytest_playwright_asyncio.pytest_playwright"):
+        raise RuntimeError(
+            "pytest-playwright and pytest-playwright-asyncio are not compatible. Please use only one of them."
+        )
     group = parser.getgroup("playwright", "Playwright")
     group.addoption(
         "--browser",
@@ -568,6 +595,7 @@ class ArtifactsRecorder:
                     page.screenshot(
                         timeout=5000,
                         path=screenshot_path,
+                        type="png",
                         full_page=self._pytestconfig.getoption(
                             "--full-page-screenshot"
                         ),
